package ach;

import java.awt.*;

/**
 * Gauge.java                                                         <br>
 * Copyright (c) 1997, 1998 Helge Hackbarth, All Rights Reserved.     <br>
 * Provides a Gauge for progress indicators. Optional cancellation.   <br>
 *                                                                    <br>
 * THE AUTHOR MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. THE AUTHOR SHALL NOT BE LIABLE FOR ANY
 * DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * @author Helge Hackbarth
 * @version 2.0
 */
public class Gauge extends Frame implements Runnable
{

  public boolean cancelled = false; // indicates if cancel button was pressed
  public static boolean visible = true; // set to false if gauge should not be displayed
  private Button cancelBtn = new Button(DemoLocal.s_cancel);
  private boolean wasPainted = false;
  private boolean updated = false;
  private long startTime;
  private GaugeCanvas gc = null;
  Panel p1 = new Panel();
  Panel p2 = new Panel();

  /**
   * Constructor for Gauge window with one line of text, gauge and optional cancel-button
   * @param title for title bar of window
   * @param msg the message to display above progress bar
   * @param min the minimum value for left side of gauge
   * @param max the maximum value for right side of gauge
   * @param canCancel set to true if gauge should contain a cancel button
   */
  public Gauge(String title, String msg, int min, int max, boolean canCancel)
  {
    super(title);
    setLayout(new BorderLayout());
    p1.add(new Label(msg));
    p1.setBackground(Color.lightGray);
    add("North", p1);
    gc = new GaugeCanvas(min, max);
    gc.setBackground(Color.lightGray);
    add("Center", gc);
    if (canCancel) {
      p2.add(cancelBtn);
      p2.setBackground(Color.lightGray);
      add("South", p2);
    }
    setBackground(Color.lightGray);
    validate();
    if ((System.getProperty("java.version").compareTo("1.1") < 0) && visible) {
    // the sequence show - resize - hide is necessary before java 1.0 can calculate correct insets
      show();
      resize(60,50);
      hide();
    }
    pack();
    int gcWidth = p1.size().width;
    if (gcWidth < 200) gcWidth = 200;
    int gcHeight = 30;
    int width = insets().left + insets().right + gcWidth;
    int height = insets().top + insets().bottom + gcHeight +
                 p1.size().height + p2.size().height;
    if (height < gcHeight + 30) height = gcHeight + 30;
    resize(width,height);
    gc.resize(gcWidth, gcHeight);
    Dimension scrSize = getToolkit().getScreenSize();
    // Problem: Toolkit of MS VM 32 from SDK 2.0beta always returns 320x240
    // so only use ScreenSize when at least VGA dimensions were returned
    if ((scrSize.width >= 640) && (scrSize.height >= 480))
    {
      move((scrSize.width-size().width) / 2, (scrSize.height - size().height) / 2);
    }
    validate();
    if (visible) {
      show();
      // setResizable(false);
      requestFocus();
    }
  }

  /**
   * Constructor for Gauge window with one line of text, gauge and cancel-button,
   * min = 0, max = 100
   * @param title for title bar of window
   * @param msg the message to display above progress bar
   */
  public Gauge(String title, String msg)
  {
    this(title, msg, 0, 100, true);
  }

  /**
   * Run-method will be called, if gauge is created and started as seperate thread
   */
  public void run()
  { }

  /**
   * overwrite handleEvent method
   */
  public boolean handleEvent(Event evt) {
    if (evt.id == Event.WINDOW_DESTROY && evt.target instanceof Gauge) {
      // do nothing
      return true;
    }
    else
    if (evt.id == Event.ACTION_EVENT) {
      if(evt.target == cancelBtn) {
        cancelled = true;
        dispose();
        return true;
      }
      else return super.handleEvent(evt);
    }
    else
    if ((evt.id == Event.KEY_PRESS) && (evt.key == 27)) { // Esc-Key
      cancelled = true;
      dispose();
      return true;
    }
    else
      return super.handleEvent(evt);
  }

  /**
   * overwrite frame's paint method
   */
  public void paint(Graphics g) {
    try {
      if (gc != null) {
        Rectangle r = gc.bounds();
        // clear GaugeGraphics only at general repaint (to avoid flickering)
        Graphics gcg = gc.getGraphics();
        gcg.clearRect(0, 0, r.width, r.height);
        gc.paint(gcg);
      gcg.dispose();
      }
     wasPainted = true;
    } catch (Throwable e) { }
    super.paint(g);
  }

  /**
   * set value for progress bar
   * @param value to display should be between min and max (see constructor)
   */
  public void setValue(int value)
  {
    if (!wasPainted)
      repaint();
    gc.setValue(value);
    if (!updated) {
      startTime = new java.util.Date().getTime();
      updated = true;
    }
  }

  /**
   * overwrite frame's dispose method
   */
  public void dispose() {
    // System.out.println("Duration: " + (double)(new java.util.Date().getTime()-startTime) / 1000 + " sec");
    super.dispose();
    if (ach.OkBox.currentOkBox != null) ach.OkBox.showCurrentOkBox();
  }

  /**
   * get current value from progress bar
   */
  public int getValue()
  { return gc.getValue(); }

}

/**
 * private classe GaugeCanvas.java                                    <br>
 * Copyright (c) 1997, 1998 Helge Hackbarth, All Rights Reserved.     <br>
 * Provides a GaugeCanvas for the Gauge class.                        <br>
 *                                                                    <br>
 * @author Helge Hackbarth
 * @version 1.4
 */
class GaugeCanvas extends Canvas
{
  int width = 200;
  int height = 30;
  int min = 0;
  int max = 100;
  int oldValue;
  int value;
  int oldScaledValue;
  public boolean wasPainted = false;

  /**
   * Constructor for GaugeCanvas
   * @param min the minimum value for left side of gauge
   * @param max the maximum value for right side of gauge
   */
  public GaugeCanvas(int min, int max) {
    super();
    this.min = min;
    this.max = max;
    oldValue = min;
    value = min;
    oldScaledValue = 0;
  }

  public void resize(int width, int height) {
    this.width = width;
    this.height = height;
    super.resize(width, height);
  }

  public void setsize(int width, int height) {
    resize(width, height);
  }

  public Dimension preferredSize() {
    return new Dimension(width, height);
  }

  public Dimension getPreferredSize() {
    return preferredSize();
  }

  public Dimension minimumSize() {
    return new Dimension(width, height);
  }

  public Dimension getMinimumSize() {
    return minimumSize();
  }

  /**
   * set value for progress bar
   * @param value to display should be between min and max (see constructor)
   */
  public void setValue(int value)
  {
    if (value != this.value) {
      oldValue = this.value;
      this.value = value;
      Graphics g = getGraphics(); paint(g); g.dispose();
    }
  }

  /**
   * get value from progress bar
   */
  public int getValue()
  { return value; }

  /**
   * overwrite canvas's paint method
   */
  public void paint(Graphics g) {
    if (g == null) return;
    try {
      int scaledValue = (int)((long)(bounds().width-10) * (value - min) / (max - min));
      int delta = scaledValue - oldScaledValue;
      if (scaledValue != oldScaledValue) {
        if (scaledValue < oldScaledValue) {
          Rectangle r = bounds();
          g.clearRect(0, 0, r.width, r.height);
        }
        if ((value > min) && (value <= max)) {
          g.setColor(Color.lightGray);
          g.draw3DRect(4, 4, (bounds().width-8), bounds().height-9, false);
          g.draw3DRect(3, 3, (bounds().width-6), bounds().height-7, false);
          g.setColor(Color.blue);
          g.fillRect(5,5,scaledValue,bounds().height-10);
        }
        oldScaledValue = scaledValue;
      }
      wasPainted = true;
    if (((delta * 100) / (max-min) > 2) && Gauge.visible) // if delta at least 2%
        ((Window)getParent()).toFront(); // make sure that gauge is visible
    } catch (Throwable e) { }
    //super.paint(g);
  }

  /**
   * overwrite canvas's update method
   */
  public void update(Graphics g) {
    // don't repaint background to avoid flickering
    try { paint(g); } catch (Throwable ne) { }
  }
}

